Attribute VB_Name = "modMain"
Option Explicit
'
' API function to open a file and get the handle
'
Public Declare Function OpenFile Lib "kernel32" _
        (ByVal lpFileName As String, lpReOpenBuff As OFSTRUCT, _
        ByVal wStyle As Long) As Long
'
' API function to close a handle
'
Public Declare Function CloseHandle Lib "kernel32" _
        (ByVal hObject As Long) As Long
'
' API function to read the file date/time stamp for all 3 date/times
'
Public Declare Function GetFileTime Lib "kernel32" _
        (ByVal hFile As Long, lpCreationTime As FILETIME, _
        lpLastAccessTime As FILETIME, lpLastWriteTime As FILETIME) As Long
'
' API function to set the file date/time stamp for all 3 date/times
'
Public Declare Function SetFileTime Lib "kernel32" (ByVal hFile As Long, _
        lpCreationTime As FILETIME, lpLastAccessTime As FILETIME, _
        lpLastWriteTime As FILETIME) As Long
'
' API function to convert a UTC file time to a local file time
'
Public Declare Function FileTimeToLocalFileTime Lib "kernel32" _
        (lpUTCFileTime As FILETIME, lpLocalFileTime As FILETIME) As Long
'
' API function to convert a local file time to a UTC file time
'
Public Declare Function LocalFileTimeToFileTime Lib "kernel32" _
        (lpLocalFileTime As FILETIME, lpUTCFileTime As FILETIME) As Long
'
' API function to convert a file time to a system time
'
Public Declare Function FileTimeToSystemTime Lib "kernel32" _
        (lpFileTime As FILETIME, lpSystemTime As SYSTEMTIME) As Long
'
' API function to convert a system time to a file time
'
Public Declare Function SystemTimeToFileTime Lib "kernel32" _
        (lpSystemTime As SYSTEMTIME, lpFileTime As FILETIME) As Long
'
' API constants
'
Public Const OF_READWRITE = &H2
Public Const HFILE_ERROR = &HFFFF
'
' API structures converted to VB:
'
'
' structure to hold file info
'
Public Type OFSTRUCT
        cBytes As Byte
        fFixedDisk As Byte
        nErrCode As Integer
        Reserved1 As Integer
        Reserved2 As Integer
        szPathName(128) As Byte
End Type
'
' structure to hold file date and time
'
Public Type FILETIME
        dwLowDateTime As Long
        dwHighDateTime As Long
End Type
'
' structure to hold system date and time
'
Public Type SYSTEMTIME
        wYear As Integer
        wMonth As Integer
        wDayOfWeek As Integer
        wDay As Integer
        wHour As Integer
        wMinute As Integer
        wSecond As Integer
        wMilliseconds As Integer
End Type
'
' global variables
'
Public FileInfoStruct As OFSTRUCT
Public CreateTime As FILETIME
Public LastAccessTime As FILETIME
Public LastWriteTime As FILETIME
Public hFile As Long
Public SysTime As SYSTEMTIME
'
' SysTime_To_String --- convert a system time to a string
'
Public Function SysTime_To_String(sTime As SYSTEMTIME) As String
     SysTime_To_String = sTime.wHour & ":" & sTime.wMinute & ":" _
            & sTime.wSecond & "." & sTime.wMilliseconds
     
End Function
'
' SysDate_To_String --- convert a system time to a string
'
Public Function SysDate_To_String(sTime As SYSTEMTIME) As String
    
     SysDate_To_String = Format(sTime.wMonth, "00") & "/" _
            & Format(sTime.wDay, "00") & "/" & sTime.wYear
     
End Function
'
' String_To_SysDateTime --- given strings containing date and time,
'                           format them into a system time variable
'                           and validate all elements
'
' a return with the year as -1 indicates an error in the date
' a return with the year as -2 indicates an error in the time
'
Public Function String_To_SysDateTime(strDate As String, _
                    strTime As String) As SYSTEMTIME
    Dim intK1 As Integer, intK2 As Integer, tempi As Integer
    '
    ' parse and validate the month
    '
    intK1 = InStr(1, strDate, "/")
    If intK1 = 0 Then
        String_To_SysDateTime.wYear = -1
        Exit Function
    Else
        tempi = CInt(Mid(strDate, 1, intK1 - 1))
        If tempi < 0 Or tempi > 12 Then
            String_To_SysDateTime.wYear = -1
            Exit Function
        Else
            String_To_SysDateTime.wMonth = tempi
        End If
    End If
    '
    ' parse and validate the day.  NOTE: day validation just checks
    ' for less than 32; a full true validation would check the
    ' correct number depending on the month
    '
    intK2 = InStr(intK1 + 1, strDate, "/")
    If intK2 = 0 Then
        String_To_SysDateTime.wYear = -1
        Exit Function
    Else
        tempi = CInt(Mid(strDate, intK1 + 1, intK2 - intK1 - 1))
        If tempi < 0 Or tempi > 31 Then
            String_To_SysDateTime.wYear = -1
            Exit Function
        Else
            String_To_SysDateTime.wDay = tempi
        End If
    End If
    '
    ' parse and validate the year --- just check that it is between 1980 and 2500
    '
    tempi = CInt(Mid(strDate, intK2 + 1))
    If tempi < 1980 Or tempi > 2500 Then
        String_To_SysDateTime.wYear = -1
        Exit Function
    Else
        String_To_SysDateTime.wYear = tempi
    End If
    '
    ' parse and validate the hours (24-hour clock)
    '
    intK1 = InStr(1, strTime, ":")
    If intK1 = 0 Then
        String_To_SysDateTime.wYear = -2
        Exit Function
    Else
        tempi = CInt(Mid(strTime, 1, intK1 - 1))
        If tempi < 0 Or tempi > 23 Then
            String_To_SysDateTime.wYear = -2
            Exit Function
        Else
            String_To_SysDateTime.wHour = tempi
        End If
    End If
    '
    ' parse and validate the minutes
    '
    intK2 = InStr(intK1 + 1, strTime, ":")
    If intK2 = 0 Then
        String_To_SysDateTime.wYear = -2
        Exit Function
    Else
        tempi = CInt(Mid(strTime, intK1 + 1, intK2 - intK1 - 1))
        If tempi < 0 Or tempi > 59 Then
            String_To_SysDateTime.wYear = -2
            Exit Function
        Else
            String_To_SysDateTime.wMinute = tempi
        End If
    End If
    '
    ' parse and validate the seconds
    '
    intK1 = InStr(intK2 + 1, strTime, ".")
    If intK1 = 0 Then
        intK1 = Len(strTime)
    Else
        tempi = CInt(Mid(strTime, intK2 + 1, intK1 - intK2 - 1))
        If tempi < 0 Or tempi > 59 Then
            String_To_SysDateTime.wYear = -2
            Exit Function
        Else
            String_To_SysDateTime.wSecond = tempi
        End If
    End If
    '
    ' NOTE: milliseconds are not parsed, so get thrown away (set to 0)
    '
End Function
